% Report Descriptive Statistics on Loan Originations and Calibrate Model Parameters 
% for "Mortgage Default during the U.S. Mortgage Crisis"
% by Thomas Schelkle
%--------------------------------------------------------
%--------------------------------------------------------

%clear all;
close all;

datapath = 'data\'; % general data path
LPSfolder = 'LPSdata\'; % sub folder containing proprietary mortgage data
graphspath = 'graphs\comp1\'; % graphs path

%--------------------------------------------------------
%--------------------------------------------------------
% Descriptive Statistics on Loan Characteristics at Origination

% Table 1 in paper (only loans with initial LTV >= 95%)
load([datapath,'FRMdata']);
sheet = 11; % LTV>=95%
LC = LC(:,:,sheet);
FICO = FICO(:,:,sheet);
InterestRate = InterestRate(:,:,sheet);
LoanAmount = LoanAmount(:,:,sheet);
LTV = LTV(:,:,sheet);

load([datapath,'LTVdistribution']);
LTVdistrib = LTVdistrib(:,:,23:end);

DTI = xlsread([datapath,LPSfolder,'DTI'],'C5:I6');
LC_LTV9599 = zeros(1,84); % LC for LTV 95-99%
LC_LTV100plus = zeros(1,84);% LC for LTV >=100%
for t = 1:84,
    LC_LTV9599(t) = sum(squeeze(LTVdistrib(1,t,1:5)));
    LC_LTV100plus(t) = sum(squeeze(LTVdistrib(1,t,6:end)));
end
LC_LTV = zeros(2,7); % yearly LC for LTV 95-99% and LTV >=100%
for year = 1:7,
    LC_LTV(:,year) = [sum(LC_LTV9599(12*(year-1)+1:12*year)),sum(LC_LTV100plus(12*(year-1)+1:12*year))];
end

cohortmeanFICO = zeros(1,8); % cohorts: 2002,...,2008 + all chorts 
cohortmeanIR = zeros(1,8);
cohortmeanLTV = zeros(1,8);
cohortmeanDTI = zeros(1,8);
cohortmeanLA = zeros(1,7);
for year = 1:7, % 2002 to 2008
    cohortmeanFICO(year) = sum(FICO(12*(year-1)+1:12*year).*LC(12*(year-1)+1:12*year))/sum(LC(12*(year-1)+1:12*year));
    cohortmeanIR(year) = sum(InterestRate(12*(year-1)+1:12*year).*LC(12*(year-1)+1:12*year))/sum(LC(12*(year-1)+1:12*year));
    cohortmeanLTV(year) = sum(LTV(12*(year-1)+1:12*year).*LC(12*(year-1)+1:12*year))/sum(LC(12*(year-1)+1:12*year));
    cohortmeanLA(year) = sum(LoanAmount(12*(year-1)+1:12*year).*LC(12*(year-1)+1:12*year))/sum(LC(12*(year-1)+1:12*year));
    cohortmeanDTI(year) = DTI(:,year)'*LC_LTV(:,year)/sum(LC_LTV(:,year));
end
cohortmeanFICO(end) = mean(cohortmeanFICO(1:7)); % average across cohorts 
cohortmeanIR(end) = mean(cohortmeanIR(1:7));
cohortmeanLTV(end) = mean(cohortmeanLTV(1:7));
cohortmeanDTI(end) = mean(cohortmeanDTI(1:7));

disp('----------------------------------------------------------------------');
disp(' ');
disp('Descriptive Statistics on Loan Originations');
disp(' ');
disp('----------------------------------------------------------------------');
disp('Table 1 in Paper: Average Loan Characteristics at Origination by Loan Cohort (initial LTV >= 95%)');
disp(['Cohort: ',num2str(2002:2008),' All'])
disp(['LTV: ',num2str(cohortmeanLTV,'% 10.1f')]);
disp(['FICO: ',num2str(cohortmeanFICO,'% 10.0f')]);
disp(['Mortgage Rate: ',num2str(cohortmeanIR,'% 10.1f')]);
disp(['DTI: ',num2str(cohortmeanDTI,'% 10.0f')]);
disp('----------------------------------------------------------------------');

% information on standard deviation of initial LTV (asked for by a referee)
LTVvec = [95:110]';
LTVdistrib=squeeze(LTVdistrib)';
cohortvarLTV = zeros(1,7);
for year = 1:7,
    cohortLTVdistrib = sum(LTVdistrib(:,12*(year-1)+1:12*year),2);
    cohortvarLTV(year) = sum((LTVvec-cohortmeanLTV(year)).^2.*cohortLTVdistrib)/sum(cohortLTVdistrib);
end
cohortstddevLTV = sqrt(cohortvarLTV);
cohortstddevLTV = [cohortstddevLTV mean(cohortstddevLTV)];
disp('Info on Standard Deviation of initial LTV by Loan Cohort (asked by a referee)');
disp(['Cohort: ',num2str(2002:2008),' All'])
disp(['Mean of LTV: ',num2str(cohortmeanLTV,'% 10.1f')]);
disp(['Std.dev. of LTV: ',num2str(cohortstddevLTV,'% 10.1f')]);
disp('----------------------------------------------------------------------');
%--------------------------------------------------------
% Table A.1 in appendix (all loans)

load([datapath,'FRMdata']);
sheet = 1; % all initial LTVs
LC = LC(:,:,sheet);
FICO = FICO(:,:,sheet);
IR = InterestRate(:,:,sheet);
LTV = LTV(:,:,sheet);

cohortmeanFICOall = zeros(1,8); % cohorts: 2002,...,2008 + all chorts 
cohortmeanIRall = zeros(1,8);
cohortmeanLTVall = zeros(1,8);
cohortmeanDTIall = zeros(1,8);
cohortmeanDTIall(1:7) = xlsread([datapath,LPSfolder,'DTI'],'C8:I8');
for year = 1:7, % 2002 to 2008
    cohortmeanFICOall(year) = sum(FICO(12*(year-1)+1:12*year).*LC(12*(year-1)+1:12*year))/sum(LC(12*(year-1)+1:12*year));
    cohortmeanIRall(year) = sum(InterestRate(12*(year-1)+1:12*year).*LC(12*(year-1)+1:12*year))/sum(LC(12*(year-1)+1:12*year));
    cohortmeanLTVall(year) = sum(LTV(12*(year-1)+1:12*year).*LC(12*(year-1)+1:12*year))/sum(LC(12*(year-1)+1:12*year));
end
cohortmeanFICOall(end) = mean(cohortmeanFICOall(1:7)); % average across cohorts 
cohortmeanIRall(end) = mean(cohortmeanIRall(1:7));
cohortmeanLTVall(end) = mean(cohortmeanLTVall(1:7));
cohortmeanDTIall(end) = mean(cohortmeanDTIall(1:7));

disp('Table 4 in Appendix: Average Loan Characteristics at Origination by Loan Cohort (All initial LTVs)');
disp(['Cohort: ',num2str(2002:2008),' All'])
disp(['LTV: ',num2str(cohortmeanLTVall,'% 10.1f')]);
disp(['FICO: ',num2str(cohortmeanFICOall,'% 10.0f')]);
disp(['Mortgage Rate: ',num2str(cohortmeanIRall,'% 10.1f')]);
disp(['DTI: ',num2str(cohortmeanDTIall,'% 10.0f')]);
disp('----------------------------------------------------------------------');

%--------------------------------------------------------
%--------------------------------------------------------
% Evidence on Default Rates for Full Sample (Appendix A)

% Figure A.1 in Appendix: Default Rates in Data for Full Sample
LTVnumb = 0; % Full Sample
backdate = 1; 
load([datapath,'FRMdata']);
Data_defcum = DefTerminations_ac_cumul(:,:,LTVnumb+1); % 60+ days default definition
Data_defcum = Data_defcum(1+backdate:end,:); % backdate default
Data_defcum(isnan(Data_defcum)) = 0;

simultime = 2002:1/12:2010+5/12; % simulation period Jan 2002 to June 2010
simultime = simultime(1:end-backdate);
Tsimul = 84; % number of origination dates used in simulation (from Jan2002 to Dec2008)
Lmax = length(simultime); % maximum length of simulation for an individual cohort 
Lind = Lmax:-1:Lmax-Tsimul+1; % simulation length for individual cohort
time02 = (1:Lmax)';
data = NaN(Lmax,7);
for ji=1:7,
    data(1:Lind(12*ji),ji) = mean(Data_defcum(1:Lind(12*ji),12*(ji-1)+1:12*ji),2);
end

figure;
colormat = [0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,100*data);
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
for i = 1:7,
    timei = time02(not(isnan(data(:,i))));
    datai = data(not(isnan(data(:,i))),i);
    text(timei(end),100*datai(end),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','bottom')
end
axis([0 100 0 100*0.14])
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
print('-depsc', [graphspath,'default_aggregate_data']);

%--------------------------------------------------------
% Decomposition of "aggregate" default rates for each cohort 

LTVvec = 0.1:0.1:1;
backdate = 1; 
simultime = 2002:1/12:2010+5/12; % simulation period Jan 2002 to June 2010
simultime = simultime(1:end-backdate);
Tsimul = 84; % number of origination dates used in simulation (from Jan2002 to Dec2008)
Lmax = length(simultime); % maximum length of simulation for an individual cohort 
Lind = Lmax:-1:Lmax-Tsimul+1; % simulation length for individual cohort
time02 = (1:Lmax)';

load([datapath,'FRMdata']);
Data_cohort_defcum = zeros(Lmax,Tsimul,length(LTVvec)); % cumulative default rates    
Data_cohort_LC = zeros(1,Tsimul,length(LTVvec));
Data_cohort_defcum_yearly = NaN(Lmax,7,length(LTVvec));
Data_aggregate_defcum_yearly = NaN(Lmax,7);
Counterf_fixeddef = NaN(Lmax,7);
Counterf_fixedcomp = NaN(Lmax,7);
Data_cohort_size_yearly = NaN(7,length(LTVvec));
fixedyear = 1; % for counterfactuals
initperiod = 1; % for taking loan counts as weights
for i = 1:length(LTVvec),
    Data_cohort_defcum(:,:,i) = DefTerminations_ac_cumul(1+backdate:end,:,i+1); % 60+ days default definition
    Data_cohort_LC(:,:,i) = LC(:,:,i+1);
    
    for ji=1:7,
        Data_cohort_defcum_yearly(1:Lind(12*ji),ji,i) = mean(Data_cohort_defcum(1:Lind(12*ji),(ji-1)*12+1:12*ji,i),2);
        Data_cohort_size_yearly(ji,i) = sum(Data_cohort_LC(1,(ji-1)*12+1:12*ji,i),2);
    end
end
for ji=1:7,
    Data_aggregate_defcum_yearly(:,ji) = squeeze(Data_cohort_defcum_yearly(:,ji,:))*Data_cohort_size_yearly(ji,:)' / sum(Data_cohort_size_yearly(ji,:));
    Counterf_fixeddef(:,ji) = squeeze(Data_cohort_defcum_yearly(:,fixedyear,:))*Data_cohort_size_yearly(ji,:)' / sum(Data_cohort_size_yearly(ji,:));
    Counterf_fixedcomp(:,ji) = squeeze(Data_cohort_defcum_yearly(:,ji,:))*Data_cohort_size_yearly(fixedyear,:)' / sum(Data_cohort_size_yearly(fixedyear,:));
end

figpos = get(0,'DefaultFigurePosition');
figure('Position',[figpos(1)/3 figpos(2) 3*figpos(3) figpos(4)]);
subplot(1,3,1);
colormat = [0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,100*Data_aggregate_defcum_yearly);
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
tit=title('True');
for i = 1:7,
    timei = time02(not(isnan(Data_aggregate_defcum_yearly(:,i))));
    datai = Data_aggregate_defcum_yearly(not(isnan(Data_aggregate_defcum_yearly(:,i))),i);
    text(timei(end),100*datai(end),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','bottom')
end
axis([0 100 0 100*0.14])
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
set(tit,'FontSize',14,'Fontweight','bold');

subplot(1,3,2);
colormat = [0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,100*Counterf_fixeddef);
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
tit=title('Fixed Default Profile');
for i = 1:7,
    timei = time02(not(isnan(Counterf_fixeddef(:,i))));
    datai = Counterf_fixeddef(not(isnan(Counterf_fixeddef(:,i))),i);
    text(timei(end),100*datai(end),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','bottom')
end
axis([0 100 0 100*0.14])
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
set(tit,'FontSize',14,'Fontweight','bold');

subplot(1,3,3);
colormat = [0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,100*Counterf_fixedcomp);
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
tit=title('Fixed Composition');
for i = 1:7,
    timei = time02(not(isnan(Counterf_fixedcomp(:,i))));
    datai = Counterf_fixedcomp(not(isnan(Counterf_fixedcomp(:,i))),i);
    text(timei(end),100*datai(end),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','bottom')
end
axis([0 100 0 100*0.14])
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
set(tit,'FontSize',14,'Fontweight','bold');


%--------------------------------------------------------
%--------------------------------------------------------
% Calibration of Parameters in Structural Model

disp(' ');
disp('Descriptive Statistics for Calibration of Structural Model');
disp(' ');
disp('----------------------------------------------------------------------');

%--------------------------------------------------------
% Section 5.1
disp('Section 5.1');

disp(['Average LTV in %: ',num2str(cohortmeanLTV(8),'% 10.1f')]);
disp(['Average Mortgage Rate in %: ',num2str(cohortmeanIR(8),'% 10.1f')]);
disp(['Average DTI in %: ',num2str(cohortmeanDTI(8),'% 10.0f')]);
disp('----------------------------------------------------------------------');

%--------------------------------------------------------
% Section 5.2
disp('Section 5.2');

% House Prices at National Level
NomHP = xlsread([datapath,'MonthlyIndex_Jan1991_to_Latest_Div'],'K2:K240');
CPI = xlsread([datapath,'CPI_series'],2,'B938:B1176');
RealHP = NomHP./CPI;
growthRealHP = log(RealHP(2:end)./RealHP(1:end-1));
Mean_growthRealHP = mean(growthRealHP);
StdDev_growthRealHP = std(growthRealHP);
disp(['Average of Monthly Real House Price Growth at National Level between 1991 and 2010 in %: ',num2str(100*Mean_growthRealHP,'% 10.3f')]);
disp(['Std.Dev. of Monthly Real House Price Growth at National Level between 1991 and 2010 in %: ',num2str(100*StdDev_growthRealHP,'% 10.2f')]);

% Volatilities at Census Divison Level
VolHP = xlsread([datapath,'vol_10q4_sales_S2'],'B1:C9');
MeanKappa = mean(VolHP(:,1));
MeanLambda = mean(VolHP(:,2));
ImpliedStdDev_IndShock1 = sqrt(MeanKappa/3+MeanLambda/9);
disp(['Average Kappa across Census Divisions: ',num2str(MeanKappa,'% 10.5f')]);
disp(['Average Lambda across Census Divisions: ',num2str(MeanLambda,'% 10.8f')]);
disp(['Implied Std.Dev. of Individual House Price Shock in First Month in %: ',num2str(100*ImpliedStdDev_IndShock1,'% 10.1f')]);
disp('----------------------------------------------------------------------');

%--------------------------------------------------------
% Section 5.3
disp('Section 5.3');

% Tax and UI Replacement Rate
MRm = (1+cohortmeanIR(1:7)/100).^(1/12) - 1; % monthly average mortgage rate 
cohortInc = zeros(1,7); % "Average" Annual Cohort Income
for year = 1:7,
    MRFactor = sum((1+MRm(year)).^(-(1:360)));
    cohortInc(year) = 12*cohortmeanLA(year)/MRFactor / (cohortmeanDTI(year)/100); 
end
taxrate = xlsread([datapath,'tax_uireplrate'],1,'B8:H8');
uireplrate = xlsread([datapath,'tax_uireplrate'],1,'B9:H9');
Meantaxrate = mean(taxrate);
Meanuireplrate = mean(uireplrate);
disp(['Average tax rate in %: ',num2str(Meantaxrate,'% 10.0f')]);
disp(['Average UI replacement rate in %: ',num2str(Meanuireplrate,'% 10.0f')]);

% Job Separation and Finding Probabilities
data = xlsread([datapath,'Unemployment'],2,'B236:C751');
dur_w = data(:,1); % median unemployment duration in weeks
dur_m = dur_w * 12 / 52; % transform duration to monthly data
rate = data(:,2); % unemployment rate in percent
time_u = 1967 + (1/12)*(5 + (1:length(dur_w))');

fm = zeros(length(dur_m),1);
for t = 1:length(dur_m),
    fm(t) = 1 - exp(1/dur_m(t) * log(0.5));
end
sm = fm.*rate/100./(1-rate/100);
Meanfm = mean(fm((time_u>=1990)&(time_u<2011)));
Meansm = mean(sm((time_u>=1990)&(time_u<2011)));
out = [time_u,dur_m,rate,fm,sm];
save([datapath,'dur_rate_fm_sm.txt'],'out','-ascii', '-tabs');

disp(['Average job finding probability between 1990 and 2010 in %: ',num2str(100*Meanfm,'% 10.0f')]);
disp(['Average job separation probability between 1990 and 2010 in %: ',num2str(100*Meansm,'% 10.1f')]);
disp('----------------------------------------------------------------------');

%--------------------------------------------------------
% Section 5.4
disp('Section 5.4');

% Real Interest Rate
TreasuryRate = xlsread([datapath,'TreasuryBill'],'C456:C701');
CPI = xlsread([datapath,'CPI_series'],2,'B925:B1171');
Infl = 100*(CPI(2:end)-CPI(1:end-1))./CPI(1:end-1);
RealRate = TreasuryRate-12*Infl;
MeanRealRate = mean(RealRate);
disp(['Average Annual Real Interest Rate between 1990 and 2010 in %: ',num2str(MeanRealRate,'% 10.1f')]);


% Inflation Rate
CPI = xlsread([datapath,'CPI_series'],2,'B1058:B1171');
Infl = (CPI(13:end)-CPI(1:end-12))./CPI(1:end-12);
MeanInfl = mean(Infl);
disp(['Average Annual Inflation Rate in Simulation Period in %: ',num2str(100*MeanInfl,'% 10.1f')]);

% Rent-Price Ratio
RentPriceData = xlsread([datapath,'DLM_rents_prices'],1,'F171:F198');
MeanRPR = mean(RentPriceData);
disp(['Average Rent-Price Ratio between 2002 and 2008 in %: ',num2str(100*MeanRPR,'% 10.1f')]);
disp('----------------------------------------------------------------------');
