% Compare value of theta to permanent income change or h.p. expectations change
% for "Mortgage Default during the U.S. Mortgage Crisis"
% by Thomas Schelkle
%--------------------------------------------------------
%--------------------------------------------------------

params;

% DTI or Average H.P. growth change?
opt_change = 1; % 1=DTI change, 2=MU change

% Initial conditions
XI0 = 0.1168+1.2657; % cash on hand
YI0 = 100; % initial house price
LI = 1; % employed
t = 1;

% theta = 0.28 case
theta=0.28;
load([resultspath,'owner_theta',num2str(1000*theta)]);
[XX,YY] = meshgrid(GPH(:,t),GX(:,t));
[XI,YI] = meshgrid(YI0,XI0);
Vinit1 = interp2(XX,YY,squeeze(V(:,:,1,LI,t)),XI,YI,interpmethsolve,-999);

% theta = 0 case
theta = 0;
load([resultspath,'owner_theta',num2str(1000*theta)]);
[XX,YY] = meshgrid(GPH(:,t),GX(:,t));
Vinit2 = interp2(XX,YY,squeeze(V(:,:,1,LI,t)),XI,YI,interpmethsolve,-999);

% check difference
if abs(Vinit1-Vinit2)<0.1,
    if opt_change == 1,
        disp(['V difference is small: ',num2str(abs(Vinit1-Vinit2))]);
        disp(['with Vinit1: ',num2str(Vinit1), ' and Vinit2: ',num2str(Vinit2)]);
        disp(['theta=0.28 equivalent to theta=0 with DTI=',num2str(DTI),', (relative income of ',num2str(0.4/DTI),')']);
    elseif opt_change == 2,
        disp(['V difference is small: ',num2str(abs(Vinit1-Vinit2))]);
        disp(['with Vinit1: ',num2str(Vinit1), ' and Vinit2: ',num2str(Vinit2)]);
        disp(['theta=0.28 equivalent to theta=0 with MU=',num2str(mu_rh_agg),', (relative MU of ',num2str(mu_rh_agg/0.00065),')']);
    end
else
    if opt_change == 1,
        disp(['V difference is still large: ',num2str(abs(Vinit1-Vinit2)), ' for DTI = ',num2str(DTI)]);
        disp(['with Vinit1: ',num2str(Vinit1), ' and Vinit2: ',num2str(Vinit2)]);
    else
        disp(['V difference is still large: ',num2str(abs(Vinit1-Vinit2)), ' for MU = ',num2str(mu_rh_agg)]);
        disp(['with Vinit1: ',num2str(Vinit1), ' and Vinit2: ',num2str(Vinit2)]);
    end
end