% Main File for Reduced Form Models
% for "Mortgage Default during the U.S. Mortgage Crisis"
% by Thomas Schelkle
% Uses: simul_rfmodel
%--------------------------------------------------------
%--------------------------------------------------------
% BEFORE RUNNING THE CODE: Create the graphspath folder on your disk!
%--------------------------------------------------------
%--------------------------------------------------------

%clear all;
close all;
ticrf = tic;

%--------------------------------------------------------
%--------------------------------------------------------
% Options and Parameters

% Paths 
comp = 1; % computation number (can be used to save results in specific folder)
datapath = 'data\';
graphspath =['graphs\comp',num2str(comp),'\'];

% Data Options 
%opt_fulldata = true; % true=have full proprietary data, false=only have public use data
defdatanumb = 60; % 60=60+delinqu,120=120+delinqu, 180=FCPresale, 200="Foreclosure completed" definition
backdate = 1; % backdate default data: 0=not, 1= by one month, 3=by three months
LTVnumb = 10; % 10=95-104% LTV bin, 8 = 75-84% LTV bin

% Reduced Form Model Parameters and Options  
frequ = 12; % frequency of the model: 12=monthly 
T = 30*frequ; % length of contract in months
PH_0 = 100; % initial price of house (normalized to 100)

opt_realcond = true; % true=base on real equity, false=base on nominal equity
opt_LTVheterog = true; % true=model within cohort LTV heterogeneity, false=use average LTV within a cohort
opt_varorig_LTV = true; % true=use variable LTV, false=use constant LTV or LTV distribution across cohorts
opt_varorig_rm = true; % true=use variable rm, false=use constant rm across cohorts
opt_hpind_dist = 1; % distribution of individual house price shocks: 1=normal, 2=uniform 
tmin = 1; % first period where default is allowed
opt_cutPt = false; %true=use phi*Pt, false=use phi*P0 in cutoff model default rule
opt_timespecpara = false; % true=use time-specific default parameter, false=constant parameter specification
opt_secondmortgage = false; % true=assume some borrowers have a second mortgage, false=no one has a second mortgage
LTVsecond = 0.15; % LTV of second mortgage
sharesecond = 0.26; % Share of borrower with a second mortgage

numi = 25000; % number of individuals in each cohort used for monte carlo simulation
cohortfit = 1; % cohort (year) used to estimate models: 1=2002,2=2003,...,8=all cohorts

defcutvec = -0.12:0.001:-0.1; % grid of parameters for estimation of model 1: threshold/cut-off model
defprobvec = 0.009:0.0001:0.011; % grid of parameters for estimation of model 2: shock model

%--------------------------------------------------------
%--------------------------------------------------------
% Load Data

% c1 = 3*12+1; % first cohort (Jan 2002)
% c2 = 10*12; % last cohort (Dec 2008)
% numcohorts = c2-c1+1; % number of cohorts 
% t_all_vec = 1:numcohorts; % all cohorts vector
% t_fit_vec = [12*(cohortfit-1)+1:12*cohortfit]; % vector of monthly cohorts used to estimate the models 
% if cohortfit == 8, % use all cohorts to estimate
%     t_fit_vec = t_all_vec;  
% end
% timevec = 1999:1/12:2010+5/12;
% timevec = timevec(1:end-backdate);
% Lind = length(timevec):-1:1; 
% Lind = Lind(c1:c2);% length of simulation period for each cohort
% Lmax = Lind(1); % maximum length of simulation period (first cohort)

c1 = 1; % first cohort (Jan 2002)
c2 = 7*12; % last cohort (Dec 2008)
numcohorts = c2-c1+1; % number of cohorts 
t_all_vec = 1:numcohorts; % all monthly cohorts vector (Jan 2002 to Dec 2008)
t_fit_vec = [12*(cohortfit-1)+1:12*cohortfit]; % vector of monthly cohorts used to estimate the models 
if cohortfit == 8, % use all cohorts to estimate
    t_fit_vec = t_all_vec;  
end
simultime = 2002:1/12:2010+5/12; % simulation period Jan 2002 to June 2010
simultime = simultime(1:end-backdate);
Lind = length(simultime):-1:1;
Lind = Lind(c1:c2);% length of simulation period for each cohort
Lmax = Lind(1); % maximum length of simulation period (first cohort)

    
% Default Rates
load([datapath,'FRMdata']);

%     time_mortg_data = time(1:end-backdate)'; % time periods of mortgage data
%     IR_data = InterestRate(2,1:length(time_mortg_data),1+LTVnumb)/100; % interest rate on new loans
%     IR_data = IR_data(c1:c2);
%     if defdatanumb == 200, % "Foreclosure completed" default definition
%         Data_defcum = FCcompleted_ac_cumul(1+backdate:length(time_mortg_data)+backdate,maxage+1:end-backdate,LTVnumb+1);
%     elseif defdatanumb ==180, % Foreclosure Presale default definition
%         Data_defcum = DefTerminationsFCPre_ac_cumul(1+backdate:length(time_mortg_data)+backdate,maxage+1:end-backdate,LTVnumb+1);
%     elseif defdatanumb ==120, % 120+ days default definition
%         Data_defcum = DefTerminations120_ac_cumul(1+backdate:length(time_mortg_data)+backdate,maxage+1:end-backdate,LTVnumb+1);
%     else % 60+ days default definition
%         Data_defcum = DefTerminations_ac_cumul(1+backdate:length(time_mortg_data)+backdate,maxage+1:end-backdate,LTVnumb+1);
%     end
%     Data_defcum(isnan(Data_defcum)) = 0;
%     defdata = Data_defcum(1:Lmax,c1:c2); % default data
%     defdatacohorts = zeros(Lind(12),7); % default data for yearly cohort
%     for ci = 1:7,
%         defdatacohorts(1:Lind(12*ci),ci) = mean(defdata(1:Lind(12*ci),12*(ci-1)+1:12*ci),2);
%     end

if defdatanumb == 200, % "Foreclosure completed" default definition
    Data_defcum = FCcompleted_ac_cumul(:,:,LTVnumb+1);
elseif defdatanumb ==180, % Foreclosure Presale default definition
    Data_defcum = DefTerminationsFCPre_ac_cumul(:,:,LTVnumb+1);
elseif defdatanumb ==120, % 120+ days default definition
    Data_defcum = DefTerminations120_ac_cumul(:,:,LTVnumb+1);
else % 60+ days default definition
    Data_defcum = DefTerminations_ac_cumul(:,:,LTVnumb+1);
end
Data_defcum = Data_defcum(1+backdate:end,:); % backdate default
Data_defcum(isnan(Data_defcum)) = 0;
defdatacohorts = zeros(Lind(12),7); % default data for yearly cohort
for ci = 1:7,
    defdatacohorts(1:Lind(12*ci),ci) = mean(Data_defcum(1:Lind(12*ci),12*(ci-1)+1:12*ci),2);
end

% Interest Rates
IR_data = InterestRate(:,:,1+LTVnumb)/100; % interest rate on new loans

% Data on Regional Composition of Mortgage Data
load([datapath,'RegionalSplit']); % contains RegSplit0
RegSplit = RegSplit0(:,:,LTVnumb+1); % regional split across state of each cohort
RegSplit(1,:) = RegSplit(1,:) - RegSplit(end,:) - RegSplit(end-1,:); %take out Unknown and Other
RegSplit = RegSplit(2:end-2,:)./repmat(RegSplit(1,:),size(RegSplit,1)-3,1);
S_to_D = load('data\States_to_Divisions.txt'); % division number of each state
S_to_D = S_to_D(2:end-2);
RegSplitDiv = zeros(9,numcohorts); % regional split across census divisions of each cohort
for t_orig = 1:numcohorts,
    for di = 1:9,
        RegSplitDiv(di,t_orig) = sum(RegSplit((S_to_D==di),t_orig));
    end
end

% LTV distributions
% load data on LTV distribution
load([datapath,'LTVdistribution']);
%LTVdistrib = squeeze(LTVdistrib(2,:,:))'; % LTV distribution at origination: LTV x cohort
%LTVdistrib = LTVdistrib(:,c1:c2);
LTVdistrib = squeeze(LTVdistrib)';
LTVdistribvec = [73:110]'; % associated values of LTVs (total,<=75,75,...,109,>=110)
if LTVnumb==10,
    LTVdistrib = LTVdistrib(LTVdistribvec>=95,:);
    LTVdistribvec = LTVdistribvec(LTVdistribvec>=95);
    % set LTV of all loans with LTV higher than 104 equal to 104
    LTVdistrib(LTVdistribvec==104,:) = sum(LTVdistrib(LTVdistribvec>=104,:));
    LTVdistrib = LTVdistrib(LTVdistribvec<=104,:);
    LTVdistribvec = LTVdistribvec(LTVdistribvec<=104);
else
    LTVdistrib = LTVdistrib((LTVdistribvec>=10*LTVnumb-5)&(LTVdistribvec<=10*LTVnumb+4),:);
    LTVdistribvec = LTVdistribvec((LTVdistribvec>=10*LTVnumb-5)&(LTVdistribvec<=10*LTVnumb+4));
end
shareLTV = LTVdistrib./repmat(sum(LTVdistrib),10,1); % LTV x cohort
    
    
% Load HPI and CPI data
hpi_cpi_data = load([datapath,'hpi_cpi_monthly_div.txt']); % US monthly Census division level nominal HPI and aggregate CPI from Jan1991 to Nov2010
hpi_time = (1991:1/12:2010+10/12)';
hpi_cpi_data = hpi_cpi_data((hpi_time>=min(simultime))&(hpi_time<=max(simultime)),:);
%hpi_cpi_data = hpi_cpi_data(97:234,:); % Jan1999 to June2010
%hpi_cpi_data = hpi_cpi_data(c1:c1+Lmax-1,:);
cpi_data = hpi_cpi_data(:,12); % aggregate CPI
hpi_nom = hpi_cpi_data(:,3:11); % nominal hpi for each division
hpi_data = hpi_nom./repmat(cpi_data,1,9); % real hpi for each division
ph_div = zeros(Lind(1),numcohorts,9); % census division house price component for each cohort
CPImat = zeros(Lind(1),numcohorts); % CPI path for each cohort 
for t_orig = 1:numcohorts,
    CPImat(1:Lind(t_orig),t_orig) = cpi_data(t_orig:t_orig+Lind(t_orig)-1)/cpi_data(t_orig);
    for di=1:9,
       ph_div(1:Lind(t_orig),t_orig,di) = hpi_data(t_orig:t_orig+Lind(t_orig)-1,di)/hpi_data(t_orig,di)*PH_0;
   end 
end

% Load division level individual house price volatility parameters
vol_div = load([datapath,'vol_div.txt']); % division level volatility estimates
kappa_div = vol_div(:,2); % division level quarterly kappa
lambda_div = vol_div(:,3); % division level quarterly lambda
var_rh_ind_div = zeros(9,Lmax);
for di=1:9,
    var_rh_ind_div(di,:) = kappa_div(di)*(4/frequ) + lambda_div(di)*(4/frequ)^2*(2*(1:Lmax)-1); % variance of individual shock depends on time
end
s_rh_ind_div = sqrt(var_rh_ind_div); % division level monthly individual house price volatility

%--------------------------------------------------------
%--------------------------------------------------------
% Prepare Simulation

% Prepare information on heterogeneity within and across cohorts 
if opt_varorig_rm, % cohort-specific mortgage rates
    rmcohort = IR_data;
else
    rmcohort = mean(IR_data)*ones(1,numcohorts);
end
if opt_LTVheterog, % heterogeneity in initial LTVs within cohort
    LTVcohort = repmat(LTVdistribvec/100,1,numcohorts);
    if not(opt_varorig_LTV),
        shareLTV = repmat(mean(shareLTV,2),1,numcohorts);
    end
else
    if opt_varorig_LTV, 
        % calculate cohort-specific average LTV
        LTVcohort = sum(shareLTV.*repmat(LTVdistribvec/100,1,numcohorts),1);
    else
        LTVcohort = mean(sum(shareLTV.*repmat(LTVdistribvec/100,1,numcohorts),1))*ones(1,numcohorts);
    end
    shareLTV = ones(1,numcohorts);
end
if opt_secondmortgage,
    LTVcohort = [LTVcohort;LTVcohort+LTVsecond];
    shareLTV = [(1-sharesecond)*shareLTV;sharesecond*shareLTV];
end

% Allocate simulation draws across census divisions and initial LTVs
% Numidiv = zeros(9,numcohorts); % regional split in simulation
% for t_orig = 1:numcohorts,
%    for di = 1:8,
%        Numidiv(di,t_orig) = round(numi*RegSplitDiv(di,t_orig));
%    end
%    Numidiv(9,t_orig) = numi - sum(Numidiv(1:8,t_orig)); 
% end
Numidiv = round(numi*RegSplitDiv); % regional split across census divisions in simulation
Numidiv(9,:) = numi*ones(1,numcohorts) - sum(Numidiv(1:8,:));
NumidivLTV = zeros(9,size(LTVcohort,1),numcohorts); % allocation of simulation draws: census division x LTVs x cohort
for t_orig = 1:numcohorts, 
    for di=1:9,
        for li=1:size(LTVcohort,1),
            if li<size(LTVcohort,1),
                NumidivLTV(di,li,t_orig) = round(Numidiv(di,t_orig)*shareLTV(li,t_orig));
            else
                if li==1,
                    NumidivLTV(di,li,t_orig) = Numidiv(di,t_orig);
                else
                    NumidivLTV(di,li,t_orig) = Numidiv(di,t_orig) - sum(NumidivLTV(di,1:li-1,t_orig));
                end
            end
        end
    end
end

%--------------------------------------------------------
%--------------------------------------------------------
% SIMULATE and ESTIMATE Models

if opt_timespecpara,
%     defnumb = zeros(Lind(c1),c2-c1+1); % number of defaulters per period in data in terms of numi (only needed if opt_timespecpara=true)
%     t_fit_vec = 1:12; % with time-specific parameters can only use the first cohort to estimate models
%     defpara_ts = NaN(Lmax,length(t_fit_vec));
%     deffrac = zeros(Lmax,length(t_fit_vec)); % fraction of defaulters per period in data 
%     for t_orig = 1:length(t_fit_vec),
%         for t=1:Lind(t_orig),
%             if t==1,
%                 deffrac(t,t_orig) = defdata(t,t_orig);
%             else
%                 deffrac(t,t_orig) = defdata(t,t_orig)-defdata(t-1,t_orig);
%             end
%             defnumb(t,t_orig) = ceil(numi*deffrac(t,t_orig));
%         end
%     end
%     defdata
%     defnumb = repmat(ceil(mean(defnumb,2)),1,length(t_fit_vec)); % replace individual default number by mean profile (for smoothing) 
    t_fit_vec = 1:12; % with time-specific parameters can only use the first cohort to estimate models
    defpara_ts = NaN(Lmax,length(t_fit_vec));
    defnumb = round(numi*(defdatacohorts(1:end,1)-[0;defdatacohorts(1:end-1,1)]));
    defnumb = [defnumb;zeros(11,1)];
    defnumb = repmat(defnumb,1,12); % number of defaulters per period in data in terms of numi
    defnumb = [defnumb,zeros(Lmax,6*12)];
end

for modelopt = 1:2,
    
    if opt_timespecpara, % use time-specific model
        defparavec = -99;
        opt_ts_det = true;
    else % use single parameter model
        if modelopt==1, 
            defparavec=defcutvec;
        elseif modelopt==2,
            defparavec=defprobvec;
        end
    end
    t_orig_vec = t_fit_vec; % during estimation only simulate for relevant in-sample cohorts
    distvec = zeros(length(defparavec),1); % distance measure to be minimized
    
    for dind = 1:length(defparavec)+1, %1:length_defpara: simulate for each defpara, then estimate, +1: simulate for estimpara
        
        if dind<=length(defparavec),
            policypara = defparavec(dind);
        elseif dind == length(defparavec)+1,
            policypara = estimpara;  
        end
        Idefcum = zeros(Lmax,numcohorts); % cumulative default rates
        
        % simulate the different cohorts in estimation and out-of-sample exercise
        for tind = 1:length(t_orig_vec),
                   
            t_orig = t_orig_vec(tind); 
            L = Lind(t_orig);
            
            CPI = CPImat(:,t_orig); % CPI path
            
            % adjust rm and LTV to cohort and LTV and generate mortgage balance
            MnLTV = zeros(T,length(LTVcohort(:,t_orig)));
            rm = rmcohort(t_orig);
            rm = (1+rm)^(1/frequ) - 1; % transform to monthly mortgage rate
            for li = 1:length(LTVcohort(:,t_orig)),    
                LTV = LTVcohort(li,t_orig);
                M_0 = LTV*PH_0; % initial loan amaount
                mn = ones(T,1)*M_0/sum((1+rm).^(-(1:T)));
                Mn = zeros(T,1);
                Mn(1) = M_0; 
                for t=2:T,
                    Mn(t) = (1+rm)*Mn(t-1) - mn(t-1);
                end
                MnLTV(:,li)=Mn;
            end
            Mn = MnLTV; % nominal mortgage balance: period x initial LTV    
    
            % set seed for random number generator
            rng(t_orig);
            
            % draw individual house price shocks
            phindshockmat = zeros(numi,L);
            for di=1:9,
                if di==1,
                    for t=1:L,
                        if opt_hpind_dist==1, % use normal distribution
                            phindshockmat(1:Numidiv(di,t_orig),t) = s_rh_ind_div(di,t)*randn(Numidiv(di,t_orig),1); %normrnd(0,s_rh_ind_div(di,t),Numidiv(di),1);
                        elseif opt_hpind_dist==2, % use uniform distribution U[-b,b]
                            b = sqrt(3) * s_rh_ind_div(di,t);
                            phindshockmat(1:Numidiv(di,t_orig),t) = -b + 2*b* rand(Numidiv(di,t_orig),1);
                        end 
                    end
                else
                    for t=1:L,
                        if opt_hpind_dist==1, % use normal distribution
                            phindshockmat(sum(Numidiv(1:di-1,t_orig))+1:sum(Numidiv(1:di-1,t_orig))+Numidiv(di,t_orig),t) = s_rh_ind_div(di,t)*randn(Numidiv(di,t_orig),1);%normrnd(0,s_rh_ind_div(di,t),Numidiv(di),1);
                        elseif opt_hpind_dist==2, % use uniform distribution U[-b,b]
                            b = sqrt(3) * s_rh_ind_div(di,t);
                            phindshockmat(sum(Numidiv(1:di-1,t_orig))+1:sum(Numidiv(1:di,t_orig)),t) = -b + 2*b* rand(Numidiv(di,t_orig),1);
                        end
                    end
                end
            end

            % individual house price component
            ph_ind = zeros(numi,L); 
            for t=1:L,
                if t==1,
                    ph_ind(:,t) = phindshockmat(:,t);
                else
                    ph_ind(:,t) = ph_ind(:,t-1) + phindshockmat(:,t);
                end
            end
            
            % aggregate house price component
            ph_agg = squeeze(ph_div(:,t_orig,:));
            ph_agg_mat = zeros(numi,L);
            for di=1:9,
                if di==1,
                    ph_agg_mat(1:Numidiv(di,t_orig),:) = repmat(ph_agg(1:L,di)',Numidiv(di,t_orig),1);
                else
                    ph_agg_mat(sum(Numidiv(1:di-1,t_orig))+1:sum(Numidiv(1:di,t_orig)),:) = repmat(ph_agg(1:L,di)',Numidiv(di,t_orig),1);
                end
            end
            
            % house price paths as combination of aggregate and individual component
            ph = zeros(numi,L); 
            for t=1:L,
                ph(:,t) = ph_agg_mat(:,t) .* exp(ph_ind(:,t));
            end
                
            % simulate default decision of models
            if opt_timespecpara,
                if modelopt==1,
                    [Idefcum(1:L,t_orig),defpara0] = simul_rfmodel(policypara,modelopt,opt_realcond,tmin,Mn,CPI,ph,Numidiv(:,t_orig),NumidivLTV(:,:,t_orig),[],opt_timespecpara,opt_ts_det,defnumb(:,t_orig),PH_0,opt_cutPt);   
                    if opt_ts_det,
                        defpara_ts(1:L,t_orig) = defpara0;
                    end
                elseif modelopt==2,
                    D = rand(numi,Lmax); % draw uniform(0,1) used to generate default shocks
                    [Idefcum(1:L,t_orig),defpara0] = simul_rfmodel(policypara,modelopt,opt_realcond,tmin,Mn,CPI,ph,Numidiv(:,t_orig),NumidivLTV(:,:,t_orig),D,opt_timespecpara,opt_ts_det,defnumb(:,t_orig),PH_0,opt_cutPt);
                    if opt_ts_det,
                        defpara_ts(1:L,t_orig) = defpara0;
                    end
                end
            else
                if modelopt==1,
                    % cut-off model
                    [Idefcum(1:L,t_orig),defpara0] = simul_rfmodel(policypara,modelopt,opt_realcond,tmin,Mn,CPI,ph,Numidiv(:,t_orig),NumidivLTV(:,:,t_orig),[],opt_timespecpara,false,[],PH_0,opt_cutPt);
                elseif modelopt==2,
                    % shock model
                    D = rand(numi,Lmax); % draw uniform(0,1) used to generate default shocks
                    [Idefcum(1:L,t_orig),defpara0] = simul_rfmodel(policypara,modelopt,opt_realcond,tmin,Mn,CPI,ph,Numidiv(:,t_orig),NumidivLTV(:,:,t_orig),D,opt_timespecpara,false,[],PH_0,opt_cutPt);
                end
            end
                      
        end % for tind
        
        if dind<=length(defparavec),
            if cohortfit == 8, % estimate on all data (in-sample exercise)
                for ji=1:7,
                    Lmin = min(Lind(12*(ji-1)+1:12*ji));
                    distvec(dind) = distvec(dind) + sum( (mean(Idefcum(1:Lmin,12*(ji-1)+1:12*ji),2) - defdatacohorts(1:Lmin,ji)).^2 );
                end
            else
                Lmin = min(Lind(t_fit_vec));
                distvec(dind) = sum( (mean(Idefcum(1:Lmin,t_fit_vec),2) - defdatacohorts(1:Lmin,cohortfit)).^2 );
            end
%             if length(t_fit_vec)==length(t_all_vec), % estimate on all data (in-sample exercise)
%                 distvec(dind) = sum( mean(Idefcum(1:min(Lind(t_orig_vec)),t_orig_vec)-defdata(1:min(Lind(t_orig_vec)),t_orig_vec),2).^2 );
%             else % estimate on part of data (out-of-sample exercise)
%                 % group by cohort year
%                 for ji=1:7,
%                     distvec(dind) = distvec(dind) + sum( mean(Idefcum(1:min(Lind(12*(ji-1)+1:12*ji)),12*(ji-1)+1:12*ji)-defdata(1:min(Lind(12*(ji-1)+1:12*ji)),12*(ji-1)+1:12*ji),2).^2 );
%                 end
%             end   
        end
        
        if dind==length(defparavec),
            if opt_timespecpara,
                estimpara = mean(defpara_ts,2);
                for ji=1:length(t_fit_vec),
                    estimpara(end-ji+1) = mean(defpara_ts(end-ji+1,1:ji));
                end
                %estimpara = median(defpara_ts,2);
                %for ji=1:length(t_fit_vec),
                %    estimpara(end-ji+1) = median(defpara_ts(end-ji+1,1:ji));
                %end
                % smoothing:
%                 estimpara0 = estimpara;
%                 estimpara = [estimpara(1:2);hpfilter(estimpara(3:90),14400);estimpara(91:end)];
%                 figure;plot([1:length(estimpara)]',estimpara0,'o',[1:length(estimpara)]',estimpara,'-');title(['default policy for model ',num2str(modelopt)]);legend('original','smoothed');
                figure;plot(estimpara);title(['default policy for model ',num2str(modelopt)]);
                opt_ts_det = false;
            else
                [mindist,indmin] = min(distvec);
                estimpara = defparavec(indmin);
                if modelopt==1,
                    defcutestim = estimpara;
                    disp(' ');
                    disp('----------------------------------------------------------------------');
                    disp(['threshold model parameter estimated as ',num2str(defcutestim),' with SSR ',num2str(mindist)]);
                    figure;plot(defparavec,distvec);ylabel('objective function');xlabel('default parameter');title('threshold model');hold on;scatter(estimpara,mindist);
                elseif modelopt==2,
                    defshockestim = estimpara;
                    disp(['shock model parameter estimated as ',num2str(defshockestim),' with SSR ',num2str(mindist)]);
                    disp('----------------------------------------------------------------------');
                    figure;plot(defparavec,distvec);ylabel('objective function');xlabel('default parameter');title('shock model');hold on;scatter(estimpara,mindist);
                end
            end
            t_orig_vec = t_all_vec; % simulate model for all cohorts for estimated parameters
             
        elseif dind==length(defparavec)+1,
            if modelopt==1,
                Idefcumcut = Idefcum;
            elseif modelopt==2,
                Idefcumshock = Idefcum;
            end
        
        end
        
    end % for dind

end % for modelopt


%--------------------------------------------------------
% Plot Results

modelcut = NaN(Lind(1),7);
modelshock = NaN(Lind(1),7);
data = NaN(Lind(1),7);
for ji=1:7,
    Lmin = min(Lind(12*(ji-1)+1:12*ji));
    modelcut(1:Lmin,ji) = mean(Idefcumcut(1:Lmin,(ji-1)*12+1:12*ji),2);
    modelshock(1:Lmin,ji) = mean(Idefcumshock(1:Lmin,(ji-1)*12+1:12*ji),2);
    data(1:Lmin,ji) = defdatacohorts(1:Lmin,ji);
end

if cohortfit<8,
%     Lfit = min(Lind(t_fit_vec));
%     datafit = mean(defdata(1:Lfit,t_fit_vec),2);
%     cutfit = mean(Idefcumcut(1:Lfit,t_fit_vec),2);
%     shockfit = mean(Idefcumshock(1:Lfit,t_fit_vec),2);
    Lfit = Lind(1);
    datafit = data(:,cohortfit);
    cutfit = modelcut(:,cohortfit);
    shockfit = modelshock(:,cohortfit);
    figure;
    set(gcf,'DefaultAxesColorOrder',[0 0 0;0.4 0.4 0.4;0.7 0.7 0.7]);
    pl=plot([1:Lfit]',100*datafit,[1:Lfit]',100*cutfit,'--',[1:Lfit]',100*shockfit,'-.');
    %tit=title(['Cumulative Default Rate of 2002 Loan Cohort: Models vs. Data']);
    xla=xlabel('months since origination');
    yla=ylabel('cumulative default rate in %');
    le=legend('data','threshold model','shock model','Location','SouthEast');
    ax=gca;
    set(pl,'Linewidth',3);
    set(ax,'FontSize',14,'Fontweight','bold');
    %set(tit,'Fontsize',14,'Fontweight','bold');
    set(xla,'Fontsize',14,'Fontweight','bold');
    set(yla,'FontSize',14,'Fontweight','bold');
    print('-depsc', [graphspath,'redform_fitted']);
end

time02 = 1:Lind(1);

figure;
colormat = [0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,100*modelcut,time02,100*data,'--');
%tit=title(['Cumulative Default Rate by Loan Cohort: Model vs. Data']);
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
%le=legend('model 2002','model 2003','model 2004','model 2005','model 2006','model 2007','model 2008','data 2002','data 2003','data 2004','data 2005','data 2006','data 2007','data 2008');
for i = 1:7,
    timei = time02(not(isnan(data(:,i))));
    datai = data(not(isnan(data(:,i))),i);
    modeli = modelcut(not(isnan(modelcut(:,i))),i);
    text(timei(end),100*max(datai(end),modeli(end)),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','bottom'); 
end
axis([0 100 0 100*(max(max(modelcut))+0.03)])
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
%set(tit,'Fontsize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
print('-depsc', [graphspath,'redform_cutoff']);

figure;
colormat = [0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0;0.4 0.4 0.4;0.7 0.7 0.7;0 0 0];
set(gcf,'DefaultAxesColorOrder',colormat);
pl=plot(time02,100*modelshock,time02,100*data,'--');
%tit=title(['Cumulative Default Rate by Loan Cohort: Model vs. Data']);
xla=xlabel('months since origination');
yla=ylabel('cumulative default rate in %');
%le=legend('model 2002','model 2003','model 2004','model 2005','model 2006','model 2007','model 2008','data 2002','data 2003','data 2004','data 2005','data 2006','data 2007','data 2008');
for i = 1:7,
   timei = time02(not(isnan(data(:,i))));
   datai = data(not(isnan(data(:,i))),i);
   modeli = modelshock(not(isnan(modelshock(:,i))),i);
   text(timei(end),100*max(datai(end),modeli(end)),num2str(2001+i),'FontSize',14,'Fontweight','bold','Color',colormat(i,:),'HorizontalAlignment','center','VerticalAlignment','bottom'); 
end
axis([0 100 0 100*(max([max(modelshock),max(data)])+0.02)]);
%axis([0 100 0 100*0.22])
ax=gca;
set(pl,'Linewidth',3);
set(ax,'FontSize',14,'Fontweight','bold');
%set(tit,'Fontsize',14,'Fontweight','bold');
set(xla,'Fontsize',14,'Fontweight','bold');
set(yla,'FontSize',14,'Fontweight','bold');
print('-depsc', [graphspath,'redform_shock']);


%--------------------------------------------------------
% Calculate GOODNESS OF FIT MEASURES


%Sum of squarred errors
SSEcut_in = sum(sum((Idefcumcut(:,t_fit_vec)-Data_defcum(:,t_fit_vec)).^2));
SSEcut_all = sum(sum((Idefcumcut(:,t_all_vec)-Data_defcum(:,t_all_vec)).^2));
SSEshock_in = sum(sum((Idefcumshock(:,t_fit_vec)-Data_defcum(:,t_fit_vec)).^2));
SSEshock_all = sum(sum((Idefcumshock(:,t_all_vec)-Data_defcum(:,t_all_vec)).^2));
%Sum of absolute errors
SAEcut_in = sum(sum(abs(Idefcumcut(:,t_fit_vec)-Data_defcum(:,t_fit_vec))));
SAEcut_all = sum(sum(abs(Idefcumcut(:,t_all_vec)-Data_defcum(:,t_all_vec))));
SAEshock_in = sum(sum(abs(Idefcumshock(:,t_fit_vec)-Data_defcum(:,t_fit_vec))));
SAEshock_all = sum(sum(abs(Idefcumshock(:,t_all_vec)-Data_defcum(:,t_all_vec))));

SSEcut_out = SSEcut_all-SSEcut_in;
SSEshock_out = SSEshock_all-SSEshock_in;
SAEcut_out = SAEcut_all-SAEcut_in;
SAEshock_out = SAEshock_all-SAEshock_in;

disp('Some Goodness of Fit Measures:')
disp(' ');
disp('Sum of Squarred Errors:   All   In-Sample   Out-of-Sample');
disp(['threshold model: ',num2str(SSEcut_all),' ',num2str(SSEcut_in),' ',num2str(SSEcut_out)]);
disp(['shock model: ',num2str(SSEshock_all),' ',num2str(SSEshock_in),' ',num2str(SSEshock_out)]);
disp(['Ratio (same for mean squarred error): ',num2str(SSEcut_all/SSEshock_all),' ',num2str(SSEcut_in/SSEshock_in),' ',num2str(SSEcut_out/SSEshock_out)]);
disp(['Ratio of root mean squarred error: ',num2str(sqrt(SSEcut_all/SSEshock_all)),' ',num2str(sqrt(SSEcut_in/SSEshock_in)),' ',num2str(sqrt(SSEcut_out/SSEshock_out))]);
disp(' ');
disp('Sum of Absolute Errors:   All   In-Sample   Out-of-Sample');
disp(['threshold model: ',num2str(SAEcut_all),' ',num2str(SAEcut_in),' ',num2str(SAEcut_out)]);
disp(['shock model: ',num2str(SAEshock_all),' ',num2str(SAEshock_in),' ',num2str(SAEshock_out)]);
disp(['Ratio (same for mean absolute error): ',num2str(SAEcut_all/SAEshock_all),' ',num2str(SAEcut_in/SAEshock_in),' ',num2str(SAEcut_out/SAEshock_out)]);
disp('----------------------------------------------------------------------');

%--------------------------------------------------------
toc(ticrf);
