% Simulate Default Decisions of a Cohort
function [Idefcum,defpara] = simul_rfmodel(policypara,modelopt,opt_realcond,tmin,Mn,CPI,ph,Numidiv,NumidivLTV,D,opt_ts,opt_det_ts,defnumb,PH_0,opt_cutPt)

numi = size(ph,1);
Lind = size(ph,2);
LTVnumber = size(NumidivLTV,2);

M = Mn(1:Lind,:)./repmat(CPI(1:Lind),1,LTVnumber);% real Mortgage balance

if opt_ts&&not(opt_det_ts), % use time-specific parameter
    defpara = policypara(1:Lind);
else % use constant parameter
    defpara = policypara(1)*ones(Lind,1);
    if modelopt==1,% only allow default for t>=tmin
        defpara(1:tmin-1) = -inf; 
    elseif modelopt==2,
        defpara(1:tmin-1) = 0; 
    end
end

if modelopt==2, 
    Defshock = D;
end

Idefcum = zeros(Lind,1); % cumulative default rate
Ihome = ones(numi,Lind); % 1=homeowner, 0=already defaulted

for t=1:Lind,  
    
    % compute equity distribution
    Mvec = zeros(numi,1);
    for di=1:9,
        if di==1,
            startd = 0;
        else
            startd = sum(Numidiv(1:di-1));
        end
        for li=1:LTVnumber,
            if li==1,
                startl = 1;
            else
                startl = sum(NumidivLTV(di,1:li-1))+1;
            end
            if opt_realcond,
                Mvec(startd+startl:startd+sum(NumidivLTV(di,1:li))) = M(t,li);
            else
                Mvec(startd+startl:startd+sum(NumidivLTV(di,1:li))) = Mn(t,li);
            end
        end
    end
    if opt_realcond,
        equ = ph(:,t)-Mvec; % real equity
    else
        equ = CPI(t)*ph(:,t) - Mvec; % nominal equity
    end
    
    % determine time-specific parameter value (if desired)
    if opt_det_ts, 
        if modelopt==1, % cutoff model
            [equsort,ixequsort] = sort(equ(Ihome(:,t)==1));
            if defnumb(t) <= 0,
                if t==1,
                    defpara(t) = -inf;
                else
                    defpara(t) = defpara(t-1);
                end
            else
                if defnumb(t)<=sum((Ihome(:,t)==1)),
                    if opt_cutPt,
                        phact = ph(Ihome(:,t)==1,t);
                        phsort = phact(ixequsort,t);
                        if opt_realcond,
                            defpara(t) = equsort(defnumb(t))/phsort(defnumb(t));
                        else
                            defpara(t) = equsort(defnumb(t))/(CPI(t)*phsort(defnumb(t)));
                        end
                    else
                        defpara(t) = equsort(defnumb(t))/PH_0;
                    end
                else
                    disp(['defnumb larger than number of active loans in period ',num2str(t)]);
                    defpara(t) = equsort(end);
                end 
            end
        elseif modelopt==2, % shock model
            negequnumb = sum(equ(Ihome(:,t)==1)<0);
            if defnumb(t) <= 0,
                if t==1,
                    defpara(t) = 0;
                else
                    defpara(t) = defpara(t-1);
                end
            else
                if negequnumb<defnumb(t),
                    defpara(t) = defpara(t-1);
                else
                    if negequnumb>0,
                        defpara(t) = defnumb(t)/negequnumb;
                    else
                        if t==1,
                            defpara(t) = 0;
                        else
                            defpara(t) = defpara(t-1);
                        end
                    end
                end  
            end     
        end
    end
    
    % generate shocks for shock model
    if modelopt==2, 
        Defshock(D(:,t)<=defpara(t),t) = 1; % 1=default shock
        Defshock(D(:,t)>defpara(t),t) = 0; % 0=no default shock
    end
    
    % determine tenure choices
    if modelopt == 1, % cut-off model
        if opt_cutPt, % use phi*Pt in default rule
            if opt_realcond,
                Isimul1 = ((Ihome(:,t)==1)&(equ < defpara(t)*ph(:,t))); % defaulters
            else
                Isimul1 = ((Ihome(:,t)==1)&(equ < defpara(t)*CPI(t)*ph(:,t))); % defaulters
            end   
        else % use phi*P0 in default rule
            Isimul1 = ((Ihome(:,t)==1)&(equ < defpara(t)*PH_0)); % defaulters
        end
    elseif modelopt == 2, % default shock model
        Isimul1 = ((Ihome(:,t)==1)&(equ < 0)&(Defshock(:,t)==1)); % defaulters
    end
        
    if t<Lind, % set next periods Ihome
        Ihome(Isimul1,t+1:end) = 0;
    end
        
    if t==1, % calculate cumulative statistics
        Idefcum(t) = sum(Isimul1)/numi;
    else
        Idefcum(t) = Idefcum(t-1)+sum(Isimul1)/numi; 
    end
end

end